/****************************************************************************
** $Id: qsdate_object.cpp  beta5   edited May 19 16:39 $
**
** Copyright (C) 2001-2003 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsdate_object.h"

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "qsoperations.h"
#include <qdatetime.h>

class QSDateShared : public QSShared
{
public:
    QDateTime dt;
};


QSDateShared *QSDateClass::dateShared( const QSObject *date )
{
    Q_ASSERT( date->objectType()->name() == "Date" );
    return (QSDateShared*) date->shVal();
}


#define get_date( obj ) QSDateClass::dateShared( obj )->dt
#define get_date_env() ( (QSDateShared*) env->thisValue().shVal() )->dt


QSDateClass::QSDateClass( QSClass *b )
    : QSSharedClass( b, AttributeFinal )
{
    addMember( "parse", QSMember( &parse, AttributeStatic ) );
    addMember( "utc", QSMember( &utc, AttributeStatic ) );

    addMember( "toString", QSMember( &toStringStatic ) );
    addMember( "toDateString", QSMember( &toDateString ) );
    addMember( "toTimeString", QSMember( &toTimeString ) );
    addMember( "valueOf", QSMember( &valueOf ) );

    addMember( "getTime", QSMember( &getTime ) );
    addMember( "getMonth", QSMember( &getMonth ) );
    addMember( "getUTCMonth", QSMember( &getMonth ) );
    addMember( "getDate", QSMember( &getDate ) );
    addMember( "getUTCDate", QSMember( &getDate ) );
    addMember( "getHours", QSMember( &getHours ) );
    addMember( "getUTCHours", QSMember( &getHours ) );
    addMember( "getMinutes", QSMember( &getMinutes ) );
    addMember( "getUTCMinutes", QSMember( &getMinutes ) );
    addMember( "getSeconds", QSMember( &getSeconds ) );
    addMember( "getUTCSeconds", QSMember( &getSeconds ) );
    addMember( "getMilliseconds", QSMember( &getMilliSeconds ) );
    addMember( "getUTCMilliseconds", QSMember( &getMilliSeconds ) );

    addMember( "setTime", QSMember( &setTime ) );
    addMember( "setMilliseconds", QSMember( &setMilliseconds ) );
    addMember( "setUTCMilliseconds", QSMember( &setMilliseconds ) );
    addMember( "setHours", QSMember( &setHours ) );
    addMember( "setSeconds", QSMember( &setSeconds ) );
    addMember( "setUTCSeconds", QSMember( &setSeconds ) );
    addMember( "setMinutes", QSMember( &setMinutes ) );
    addMember( "setUTCMinutes", QSMember( &setMinutes ) );
    addMember( "setDate", QSMember( &setDate ) );
    addMember( "setUTCDate", QSMember( &setDate ) );
    addMember( "setMonth", QSMember( &setMonth ) );
    addMember( "setUTCMonth", QSMember( &setMonth ) );
    addMember( "setYear", QSMember( &setYear ) );

    addMember( "getYear", QSMember( &getYear ) );
    addMember( "toGMTString", QSMember( &toGMTString ) );
}

QSObject QSDateClass::construct( const QSList &args ) const
{
    QSDateShared *shared = new QSDateShared;
    int numArgs = args.size();
    if( numArgs == 0 ) {
	shared->dt = QDateTime::currentDateTime();
    }
    if( numArgs == 1 ) {
	QSObject arg = args[0];
	if( arg.isString() ) {
	    shared->dt = QDateTime::fromString( arg.toString() );
	} else {
	    shared->dt.setTime_t( (uint) arg.toNumber() );
	}
    } else if ( numArgs>1 ) {
	int year  = args[0].toInt32();
	int month = args[1].toInt32();
	int day   = numArgs >= 3 ? args[2].toInt32() : 1;
	int hour  = numArgs >= 4 ? args[3].toInt32() : 0;
	int min   = numArgs >= 5 ? args[4].toInt32() : 0;
	int sec   = numArgs >= 6 ? args[5].toInt32() : 0;
	int milli = numArgs >= 7 ? args[6].toInt32() : 0;
	shared->dt.setDate( QDate( year, month, day ) );
	shared->dt.setTime( QTime( hour, min, sec, milli ) );
    }

    return QSObject( this, shared );
}

QSObject QSDateClass::cast( const QSList &args ) const
{
    return construct( args );
}

bool QSDateClass::toBoolean( const QSObject *obj ) const
{
    return get_date( obj ).isValid(); // ### Correct assumtion?
}

double QSDateClass::toNumber( const QSObject *obj ) const
{
    return get_date( obj ).toTime_t();
}

QString QSDateClass::toString( const QSObject *o ) const
{
    return get_date( o ).toString();
}


QVariant QSDateClass::toVariant( const QSObject *obj, QVariant::Type t ) const
{
    switch( t ) {
    case QVariant::Time:
	return get_date( obj ).time();
    case QVariant::Date:
	return get_date( obj ).date();
    default:
	return get_date( obj );
    }
}


// Date.parse()
QSObject QSDateClass::parse( QSEnv *env )
{
    if ( env->arg( 0 ).isA( env->stringClass() ) ) {
	QSObject date( env->dateClass() );
	QSDateShared *shared = new QSDateShared();
	shared->dt = QDateTime::fromString( env->arg( 0 ).toString(), Qt::ISODate );
	date.setVal( shared );
	return date;
    }
    return env->createUndefined();
}


// Date.UTC()
QSObject QSDateClass::utc( QSEnv *env )
{
    const QSList *args = env->arguments();
    int numArgs = args->size();
    if ( numArgs >= 2 ) {
	int year  = args->at( 0 ).toInt32();
	int month = args->at( 1 ).toInt32();
	int day   = numArgs >= 3 ? args->at( 2 ).toInt32() : 1;
	int hour  = numArgs >= 4 ? args->at( 3 ).toInt32() : 0;
	int min   = numArgs >= 5 ? args->at( 4 ).toInt32() : 0;
	int sec   = numArgs >= 6 ? args->at( 5 ).toInt32() : 0;
	int milli = numArgs >= 7 ? args->at( 6 ).toInt32() : 0;
	QDateTime dt;
	dt.setDate( QDate( year, month, day ) );
	dt.setTime( QTime( hour, min, sec, milli ) );
	return env->createNumber( dt.toTime_t() );
    }
    return env->createUndefined();
}

QSObject QSDateClass::toStringStatic( QSEnv *env )
{
    return env->createString( get_date_env().toString() );
}

QSObject QSDateClass::toDateString( QSEnv *env )
{
    return env->createString( get_date_env().toString() );
}

QSObject QSDateClass::toTimeString( QSEnv *env )
{
    return env->createString( get_date_env().toString() );
}

QSObject QSDateClass::toGMTString( QSEnv *env )
{
    return env->createString( get_date_env().toString() );
}

QSObject QSDateClass::valueOf( QSEnv *env )
{
    return env->createNumber( get_date_env().toTime_t() );
}

QSObject QSDateClass::getTime( QSEnv *env )
{
    QSObject obj = env->thisValue();
    return env->createNumber( get_date_env().toTime_t() );
}

QSObject QSDateClass::getYear( QSEnv *env )
{
    return env->createNumber( get_date_env().date().year() );
}

QSObject QSDateClass::getMonth( QSEnv *env )
{
    return env->createNumber( get_date_env().date().month() );
}

QSObject QSDateClass::getDate( QSEnv *env )
{
    return env->createNumber( get_date_env().date().day() );
}

QSObject QSDateClass::getHours( QSEnv *env )
{
    return env->createNumber( get_date_env().time().hour() );
}

QSObject QSDateClass::getMinutes( QSEnv *env )
{
    return env->createNumber( get_date_env().time().minute() );
}

QSObject QSDateClass::getSeconds( QSEnv *env )
{
    return env->createNumber( get_date_env().time().second() );
}

QSObject QSDateClass::getMilliSeconds( QSEnv *env )
{
    return env->createNumber( get_date_env().time().msec() );
}

QSObject QSDateClass::setTime( QSEnv *env )
{
    get_date_env().setTime_t( (uint) env->arg( 0 ).toNumber() );
    return env->thisValue();
}


QSObject QSDateClass::setMilliseconds( QSEnv *env )
{
    QDateTime &dt = get_date_env();
    QTime time = dt.time();
    dt.setTime( QTime( time.hour(),
		       time.minute(),
		       time.second(),
		       (int) env->arg(0).toNumber() ) );
    return env->thisValue();
}


QSObject QSDateClass::setSeconds( QSEnv *env )
{
    QDateTime &dt = get_date_env();
    QTime time = dt.time();
    dt.setTime( QTime( time.hour(),
		       time.minute(),
		       (int) env->arg(0).toNumber(),
		       time.msec() ) );
    return env->thisValue();
}


QSObject QSDateClass::setMinutes( QSEnv *env )
{
    QDateTime &dt = get_date_env();
    QTime time = dt.time();
    dt.setTime( QTime( time.hour(),
		       (int) env->arg( 0 ).toNumber(),
		       time.second(),
		       time.msec() ) );
    return env->thisValue();
}


QSObject QSDateClass::setHours( QSEnv *env )
{
    QDateTime &dt = get_date_env();
    QTime time = dt.time();
    dt.setTime( QTime( (int) env->arg( 0 ).toNumber(),
		       time.minute(),
		       time.second(),
		       time.msec() ) );
    return env->thisValue();
}


QSObject QSDateClass::setDate( QSEnv *env )
{
    QDateTime &dt = get_date_env();
    QDate date = dt.date();
    dt.setDate( QDate( date.year(),
		       date.month(),
		       (int) env->arg( 0 ).toNumber() ) );
    return env->thisValue();
}

QSObject QSDateClass::setMonth( QSEnv *env )
{
    QDateTime &dt = get_date_env();
    QDate date = dt.date();
    dt.setDate( QDate( date.year(),
		       (int) env->arg( 0 ).toNumber(),
		       date.day() ) );
    return env->thisValue();
}

QSObject QSDateClass::setYear( QSEnv *env )
{
    QDateTime &dt = get_date_env();
    QDate date = dt.date();
    dt.setDate( QDate( (int) env->arg( 0 ).toNumber(),
		       date.month(),
		       date.day() ) );
    return env->thisValue();
}
