/****************************************************************************
** $Id: qsstring_object.cpp  beta5   edited Feb 11 18:08 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsstring_object.h"
#include "qsregexp_object.h"
#include "qsoperations.h"
#include "qstypes.h"
#include "qsclass.h"
#include "qsenv.h"
#include <qregexp.h>

using namespace QS;

QSStringClass::QSStringClass( QSClass *b )
    : QSClass( b, AttributeFinal )
{
}

void QSStringClass::init()
{
    // custom property
    addMember( "length", QSMember( QSMember::Custom, 0,
				   AttributeNonWritable ) );

    // static function
    addMember( "fromCharCode", QSMember( &fromCharCode, AttributeStatic ) );

    // member functions
    addMember( "toString", QSMember( &QSStringClass::toStringScript ) );
    addMember( "valueOf", QSMember( &QSStringClass::toStringScript ) );
    addMember( "charAt", QSMember( &charAt ) );
    addMember( "charCodeAt", QSMember( &charCodeAt ) );
    addMember( "indexOf", QSMember( &indexOf ) );
    addMember( "lastIndexOf", QSMember( &lastIndexOf ) );
    addMember( "match", QSMember( &match ) );
    addMember( "search", QSMember( &search ) );
    addMember( "replace", QSMember( &replace ) );
    addMember( "split", QSMember( &split ) );
    addMember( "substr", QSMember( &substr ) );
    addMember( "substring", QSMember( &substring ) );
    addMember( "toLowerCase", QSMember( &toLowerCase ) );
    addMember( "lower", QSMember( &toLowerCase ) );
    addMember( "toUpperCase", QSMember( &toUpperCase ) );
    addMember( "upper", QSMember( &toUpperCase ) );
    addMember( "isEmpty", QSMember( &isEmpty ) );
    addMember( "left", QSMember( &left ) );
    addMember( "mid", QSMember( &mid ) );
    addMember( "right", QSMember( &right ) );
    addMember( "find", QSMember( &find ) );
    addMember( "findRev", QSMember( &findRev ) );
    addMember( "startsWith", QSMember( &startsWith ) );
    addMember( "endsWith", QSMember( &endsWith ) );
}

void QSStringClass::ref( QSObject *o ) const
{
    o->val.str->ref();
}

void QSStringClass::deref( QSObject *o ) const
{
    o->val.str->deref();
    if ( o->val.str->count == 0 )
	delete o->val.str;
}

QSObject QSStringClass::fetchValue( const QSObject *objPtr,
				    const QSMember &mem ) const
{
    if ( mem.type() == QSMember::Custom )
	if ( mem.idx == 0 ) {
	    return createNumber( objPtr->sVal().length() );
	} else {
	    qFatal( "QSStringClass::fetchValue: unhandled member" );
	    return createUndefined();
	}
    else
	return QSClass::fetchValue( objPtr, mem );
}

bool QSStringClass::toBoolean( const QSObject *obj ) const
{
    return !obj->sVal().isEmpty();
}

double QSStringClass::toNumber( const QSObject *obj ) const
{
    return QSString::toDouble( obj->sVal() );
}

QString QSStringClass::toString( const QSObject *obj ) const
{
    return obj->sVal();
}

QSObject QSStringClass::toPrimitive( const QSObject *obj,
				     const QSClass * ) const
{
    return *obj;
}

QVariant QSStringClass::toVariant( const QSObject *obj, QVariant::Type ) const
{
    return obj->sVal();
}

QSObject QSStringClass::construct( const QSList &args ) const
{
    QString s = args.isEmpty() ? QString( "" ) : args[0].toString();

    return createString( s );
}

/*!
  \reimp
*/
QSObject QSStringClass::cast( const QSList &args ) const
{
    return construct( args );
}

QSEqualsResult QSStringClass::isEqual( const QSObject &a, const QSObject &b ) const
{
    Q_ASSERT( a.isA( this ) );
    if ( b.isString() )
	return ( QSEqualsResult ) ( a.sVal() == b.sVal() );
    else if ( b.isNumber() )
	return ( QSEqualsResult ) ( a.sVal() == b.toString() );
    else if ( !b.isPrimitive() )
	return isEqual( a, b.toPrimitive() );
    else
	return EqualsUndefined;

}

// String.fromCharCode()
QSObject QSStringClass::fromCharCode( QSEnv *env )
{
    const QSList *args = env->arguments();
    QString s;
    if ( args->size() ) {
	s.setLength( args->size() );
	QSListIterator it = args->begin();
	int i = 0;
	while ( it != args->end() ) {
	    ushort u = it->toUInt16();
	    s[i] = QChar( u );
	    it++;
	    i++;
	}
    } else {
	s = "";
    }

    return env->createString( s );
}

QSObject QSStringClass::toStringScript( QSEnv *env )
{
    return env->createString( env->thisValue().sVal() );
}

QSObject QSStringClass::charAt( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    int pos = env->arg( 0 ).toInteger();
    if ( pos < 0 || pos >= (int)s.length() )
	s = "";
    else
	s = s.mid( pos, 1 );
    return env->createString( s );
}

QSObject QSStringClass::charCodeAt( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    int pos = env->arg( 0 ).toInteger();
    double d;
    if ( pos < 0 || pos >= (int)s.length())
	d = NaN;
    else
	d = s[pos].unicode();
    return env->createNumber( d );
}

QSObject QSStringClass::indexOf( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    QString s2 = env->arg( 0 ).toString();
    int pos = env->numArgs() >= 2 ? env->arg( 1 ).toInteger() : 0;
    if ( pos < 0 )
	pos = 0;
    return env->createNumber( s.find( s2, pos ) );
}

QSObject QSStringClass::lastIndexOf( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    QString s2 = env->arg( 0 ).toString();
    int pos = env->numArgs() >= 2 ? env->arg( 1 ).toInteger() : s.length();
    return env->createNumber( s.findRev( s2, pos ) );
}

QSObject QSStringClass::match( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    if ( env->arg( 0 ).typeName() == "RegExp" ) {
	QString s2 = env->arg( 0 ).get( "source" ).toString();
	QRegExp reg( s2 );
	reg.search( s );
	env->regexpClass()->lastCaptures = reg.capturedTexts();
	QString mstr = reg.cap();
	if ( mstr.isNull() )
	    return env->createNull();
	// ### return an array, with the matches
	return env->createString( mstr );
    } else {
	return env->createUndefined();
    }
}

QSObject QSStringClass::search( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    if ( env->arg( 0 ).typeName() == "RegExp" ) {
	QString s2 = env->arg( 0 ).get("source").toString();
	QRegExp reg( s2 );
	int pos = reg.search( s );
	env->thisValue().env()->regexpClass()->lastCaptures = reg.capturedTexts();
	return env->createNumber( pos );
    } else {
	return env->createUndefined();
    }
}

QSObject QSStringClass::replace( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    int pos, len;
    // ### only most common case
    if ( env->arg( 0 ).typeName() == "RegExp" ) {
	QString s2 = env->arg( 0 ).get("source").toString();
	QRegExp reg( s2 );
	pos = reg.search( s );
	len = reg.matchedLength();
    } else {
	QString s2 = env->arg( 0 ).toString();
	pos = s.find( s2 );
	len = s.length();
    }
    if ( pos == -1 )
	return env->createString( s );
    QString r = s.mid( 0, pos) + env->arg( 1 ).toString() + s.mid( pos + len );
    return env->createString( r );
}

QSObject QSStringClass::split( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    QSArray result( env );
    int i = 0, p0 = 0;
    double d = env->numArgs() >= 2 ? env->arg( 1 ).toInteger() : -1; // optional max
    if ( env->arg( 0 ).isA( "RegExp" ) ) {
	QRegExp reg( env->arg( 0 ).get("source").toString() );
	if ( s.isEmpty() && reg.search( s, 0 ) >= 0) {
	    // empty string matched by regexp -> empty array
	    result.put( "length", env->createNumber( 0 ) );
	    return result;
	}
	int pos = 0;
	for ( ;; ) {
	    /* TODO: back references */
	    int mpos = reg.search( s, pos );
	    if ( mpos < 0 )
		break;
	    QString mstr = reg.cap( 0 );
	    pos = mpos + ( mstr.isEmpty() ? 1 : mstr.length() );
	    if ( mpos != p0 || !mstr.isEmpty() ) {
		result.put( QString::number( i ),
			    env->createString( s.mid( p0, mpos-p0 ) ));
		p0 = mpos + mstr.length();
		i++;
	    }
	}
    } else if ( env->numArgs() >= 1 ) {
	QString u2 = env->arg( 0 ).toString();
	if ( u2.isEmpty() ) {
	    if ( s.isEmpty() ) {
		// empty separator matches empty string -> empty array
		result.put( "length", env->createNumber( 0 ) );
		return result;
	    } else {
		while ( i != d && i < (int)s.length() )
		    result.put( QString::number( i++ ),
				env->createString( s.mid( p0++, 1 ) ) );
	    }
	} else {
	    int pos;
	    while (i != d && (pos = s.find(u2, p0)) >= 0) {
		result.put( QString::number( i ),
			    env->createString( s.mid( p0, pos-p0 ) ) );
		p0 = pos + u2.length();
		i++;
	    }
	}
    }
    // add remaining string, if any
    if ( i != d && (p0 < (int)s.length() || i == 0) )
	result.put( QSString::from( i++ ), env->createString( s.mid( p0 ) ) );
    result.put( "length", i );
    return result;
}

QSObject QSStringClass::substr( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    uint len = s.length();
    int n = env->arg( 0 ).toInteger();
    int m = env->arg( 1 ).toInteger();
    double d, d2;
    if ( n >= 0 )
	d = n;
    else
	d = QMAX(len + n, 0);
    if ( env->numArgs() < 2 )
	d2 = len - d;
    else
	d2 = min( QMAX( m, 0 ), len - d );
    return env->createString( s.mid( (int)d, (int)d2 ) );
}

QSObject QSStringClass::substring( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    uint len = s.length();
    int n = env->arg( 0 ).toInteger();
    int m = env->arg( 1 ).toInteger();
    double d = QMIN( QMAX( n, 0 ), (int)len );
    double d2;
    if ( env->numArgs() < 2 ) {
	d2 = len - d;
    } else {
	d2 = min( max( m, 0), len );
	d2 = max(d2-d, 0);
    }
    return env->createString( s.mid((int)d, (int)d2 ) );
}

QSObject QSStringClass::toLowerCase( QSEnv *env )
{
    return env->createString( env->thisValue().sVal().lower() );
}

QSObject QSStringClass::toUpperCase( QSEnv *env )
{
    return env->createString( env->thisValue().sVal().upper() );
}

QSObject QSStringClass::isEmpty( QSEnv *env )
{
    return env->createBoolean( env->thisValue().sVal().isEmpty() );
}

QSObject QSStringClass::left( QSEnv *env )
{
    return env->createString( env->thisValue().sVal().left( env->arg( 0 ).toInteger() ) );
}

QSObject QSStringClass::mid( QSEnv *env )
{
    int len = -1;
    if ( env->arg( 1 ).isNumber() )
	len = env->arg( 1 ).toInteger();
    return env->createString( env->thisValue().sVal().mid( env->arg( 0 ).toInteger(), len ) );
}

QSObject QSStringClass::right( QSEnv *env )
{
    return env->createString( env->thisValue().sVal().right( env->arg( 0 ).toInteger() ) );
}

QSObject QSStringClass::find( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    // ### regexp overload
    int idx = env->numArgs() >= 2 ? env->arg( 1 ).toInteger() : 0;
    bool cs = env->numArgs() >= 3 ? env->arg( 2 ).toBoolean() : TRUE;
    return env->createNumber( s.find( env->arg( 0 ).toString(), idx, cs ) );
}

QSObject QSStringClass::findRev( QSEnv *env )
{
    QString s = env->thisValue().sVal();
    // ### regexp overload
    int idx = env->numArgs() >= 2 ? env->arg( 1 ).toInteger() : -1;
    bool cs = env->numArgs() >= 3 ? env->arg( 2 ).toBoolean() : TRUE;
    return env->createNumber( s.findRev( env->arg( 0 ).toString(), idx, cs ) );
}

QSObject QSStringClass::startsWith( QSEnv *env )
{
    return env->createBoolean( env->thisValue().sVal().startsWith( env->arg( 0 ).toString() ) );
}

QSObject QSStringClass::endsWith( QSEnv *env )
{
    return env->createBoolean( env->thisValue().sVal().endsWith( env->arg( 0 ).toString() ) );
}
