#include "parser.h"

#include <qfile.h>
#include <qstring.h>

QString NameSpaceParser::legalChars = "abcdefghijklmnopqrstufvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_/*";

NameSpaceParser::NameSpaceParser()
{
}

void NameSpaceParser::parse( const QString &in_file, const QString &out_file )
{
    QFile infile( in_file );
    if ( !infile.open( IO_ReadOnly ) )
	return;
    QFile outfile( out_file );
    if ( !outfile.open( IO_WriteOnly ) )
	return;

    QTextStream outstream( &outfile );
    outstream << "#ifndef QUICKNAMESPACE_H" << endl;
    outstream << "#define QUICKNAMESPACE_H" << endl << endl;
    outstream << "#include \"quickdispatchobject.h\"" << endl << endl;
    outstream << "class QuickNamespace : public QuickUnnamedObject" << endl;
    outstream << "{" << endl;
    outstream << "    Q_OBJECT" << endl << endl;

    appendLater += "public:\n";
    appendLater += "    QuickNamespace() {}\n";

    QString contents = infile.readAll();

    QString text;
    for ( int i = 0; i < (int)contents.length(); ++i ) {
	if ( legalChars.find( contents[i] ) != -1 )
	    text += contents[i];
	else
	    text = "";
	if ( text == "enum" )
	    parseEnum( i, contents, outstream );
    }

    outstream << appendLater << endl;
    outstream << "};" << endl;
    outstream << "#endif" << endl;
}

void NameSpaceParser::parseEnum( int &i, const QString &contents, QTextStream &outstream )
{
    bool inEnum = FALSE;
    enum State { BeforeName, InName, AfterName, BeforeVaue, InValue, AfterValue };
    int state;
    QString name, value;
    int currValue = -1;
    bool writeIt = FALSE;
    for ( ; i < (int)contents.length(); ++i ) {
	if ( contents[i] == '/' && contents[i+1] == '/') {
	    while ( contents[i] != '\n' && i < (int)contents.length() )
		++i;
	    continue;
	}
	if ( contents[i] == '{' ) {
	    inEnum = TRUE;
	    state = BeforeName;
	    name = value = "";
	    continue;
	}
	if ( inEnum ) {
	    writeIt = FALSE;
	    switch ( state ) {
	    case BeforeName:
		if ( legalChars.find( contents[i] ) != -1 ) {
		    state = InName;
		    name = contents[i];
		}
		break;
	    case InName:
		if ( legalChars.find( contents[i] ) == -1 ) {
		    state = AfterName;
		    i--;
		} else {
		    name += contents[i];
		}
		break;
	    case AfterName:
		if ( contents[i] == '=' )
		    state = BeforeVaue;
		else if ( contents[i] == ',' || contents == '}' ) {
		    writeIt = TRUE;
		    state = BeforeName;
		}
		break;
	    case BeforeVaue:
		if ( legalChars.find( contents[i] ) != -1 ) {
		    state = InValue;
		    value = contents[i];
		}
		break;
	    case InValue:
		if ( legalChars.find( contents[i] ) == -1 ) {
		    state = AfterValue;
		    i--;
		} else {
		    value += contents[i];
		}
		break;
	    case AfterValue:
		if ( contents[i] == ',' || contents == '}' ) {
		    state = BeforeName;
		    writeIt = TRUE;
		}
		break;
	    }
	    if ( writeIt && name != "//" && name != "ifdef" &&
		 name != "else" && name != "endif" && name != "elseif" &&
		 name != "Pixmap" && name != "System" &&
		 knownValues.find( name ) == knownValues.end() ) {
		outstream << "    Q_PROPERTY( int " << name << " READ " << name << " )" << endl;
		if ( value.simplifyWhiteSpace().isEmpty() )
		    value = QString::number( ++currValue );
		else
		    currValue = value.toInt();
		if ( knownValues.find( QString( value ) ) != knownValues.end() )
		    value += "()";
		knownValues.replace( name, name );
		appendLater += "    int " + name + "() const { return " + value + "; }\n";
		value = "";
	    }
	    if ( contents[i] == '}' ) {
		++i;
		return;
	    }
	}
    }
}
