#include <qsinterpreter.h>
#include <qsproject.h>
#include <qsscript.h>

#include <qapplication.h>
#include <qfile.h>
#include <qfileinfo.h>

static const char *appName;

void help()
{
    printf( "USAGE:\n"
	    "  %s -create <projectname> <qsfile> <qsfile> ...\n"
	    "  %s -expand <projectfile>\n"
	    "  %s -display <projectfile>\n",
	    appName, appName, appName );
}

#define exitOn( x ) if( argc<(x) ) { help(); return 0; }

QString load( const QString &filename )
{
    QFile file( filename );
    if( !file.open( IO_ReadOnly ) ) {
	printf( "Could not open file: %s\n", filename.latin1() );
	return QString::null;
    }
    return QString( file.readAll() );
}

void write( const QString &filename, const QString &code )
{
    QFile file( filename );
    if( !file.open( IO_WriteOnly ) ) {
	printf( "Could not open file for writing: %s\n", filename.latin1() );
	return;
    }
    file.writeBlock( code.latin1(), code.length() );
    file.flush();
    file.close();

    printf( "wrote file: %s", filename.latin1() );
}

int main( int argc, char **argv )
{
    QApplication app( argc, argv );

    appName = argv[0];
    exitOn( 2 );

    QSProject *project = new QSProject();

    if( argv[1] == QString( "-create" ) ) {
	exitOn( 4 );
	printf( "Creating project::\n" );
	QFile( argv[2] ).remove();
	project->load( argv[2] );
	for( int i=3; i<argc; i++ ) {
	    QString script = load( argv[i] );
	    if( !script.isNull() ) {
		printf( " -> adding script: %s\n",
			QFileInfo( argv[i] ).fileName().latin1() );
		project->createScript( QFileInfo( argv[i] ).fileName(),
				       script );
	    }
	}
	project->save( argv[2] );
    } else if( argv[1] == QString( "-expand" ) ) {
	exitOn( 3 );
	printf( "Expanding project::\n" );
	project->load( argv[2] );
	QPtrList<QSScript> scripts = project->scripts();
	for( int i=0; i<int(scripts.count()); i++ ) {
	    QSScript *script = scripts.at( i );
	    write( script->name(), script->code() );
	    printf( " -> expanded script: %s\n", script->name().latin1() );
	}
    } else if( argv[1] == QString( "-display" ) ) {
	exitOn( 3 );
	project->load( argv[2] );
	QPtrList<QSScript> scripts = project->scripts();
	for( int i=0; i<int(scripts.count()); i++ ) {
	    QSScript *script = scripts.at( i );
	    printf( "********** BEGIN SCRIPT: %s\n%s\n********** END SCRIPT %s\n\n",
		    script->name().latin1(),
		    script->code().latin1(),
		    script->name().latin1() );
	}
	printf( "Number of scripts: %d\n", scripts.count() );
	for( int i=0; i<int(scripts.count()); i++ ) {
	    printf( " - %s\n", scripts.at(i)->name().latin1() );
	}
    } else {
	exitOn( argc );
    }
    return 0;
}
